library shelf.handlers.file;

import 'dart:async';
import 'dart:io';
import 'package:path/path.dart' as pathos;
import '../shared.dart';

ShelfHandler fileHandler(String path) {
  var handler = new _FileHandler(path);
  return handler.handle;
}

class _FileHandler {
  final String _rootPath;

  _FileHandler(String path) :
    this._rootPath = pathos.absolute(path);

  dynamic handle(HttpRequest request, Future<ShelfResponse> next()) {
    // TODO: only GET

    var requestPath = pathos.joinAll(request.uri.pathSegments);

    var mappedPath = pathos.join(_rootPath, requestPath);
    // NEVER serve a file outside of the mappedPath
    if(!pathos.isWithin(_rootPath, mappedPath)) return null;

    var file = new File(mappedPath);
    return file.exists()
        .then((bool exists) {
          if(!exists) return null;

          return _getResponse(request, file);
        });
  }

  Future<ShelfResponse> _getResponse(HttpRequest request, File file) {
    assert(file.existsSync());

    // mime type
    // size
    // etag support

    // we'll get a lot more fancy later, but for now...
    return file.readAsString()
        .then((String body) {
          return new ShelfResponse(200, const {}, body);
        });
  }
}
