library harness_console;

import 'dart:io';
import 'package:unittest/unittest.dart';

import 'package:shelf/shelf.dart';
import '../shared.dart';

void main() {
  test('default should be 404', () {
    return startShelf()
        .handleRequest(const RequestMock())
        .then((ShelfResponse response) {
          expect(response.code, 404);
          expect(response.body, 'Could not find a resource for "/"');
        });
  });

  group('root and foo', () {
    Shelf shelf;


    setUp(() {
      shelf = startShelf()
          ..add(_rootHandler)
          ..add(_fooHandler);
    });

    test('root', () {
      return shelf.handleRequest(const RequestMock())
          .then((ShelfResponse response) {
            expect(response.code, 200);
            expect(response.body, "hello from root");
          });
    });

    test('foo/test_data', () {
      return shelf.handleRequest(const RequestMock(path: 'foo/test_data'))
          .then((ShelfResponse response) {
            expect(response.code, 200);
            expect(response.body, "foo:/test_data");
          });
    });

    test('foo', () {
      return shelf.handleRequest(const RequestMock(path: 'foo'))
          .then((ShelfResponse response) {
            expect(response.code, 200);
            expect(response.body, "foo:");
          });
    });

    test('foo/', () {
      return shelf.handleRequest(const RequestMock(path: 'foo/'))
          .then((ShelfResponse response) {
            expect(response.code, 200);
            expect(response.body, "foo:/");
          });
    });

    test('bar', () {
      return shelf.handleRequest(const RequestMock(path: 'bar'))
          .then((ShelfResponse response) {
            expect(response.code, 404);
            expect(response.body, 'Could not find a resource for "bar"');
          });
    });
  });

}

ShelfResponse _rootHandler(HttpRequest request, next) {
  if(request.uri.pathSegments.isEmpty) {
    return new ShelfResponse(200, const {}, 'hello from root');
  }
}

ShelfResponse _fooHandler(HttpRequest request, next) {
  var segs = request.uri.pathSegments;
  if(segs.isNotEmpty && segs.first == 'foo') {
    var rest = request.uri.path.substring(3);
    return new ShelfResponse(200, const {}, 'foo:$rest');
  }
}
