library test.shelf.file_handler;

import 'dart:io';
import 'package:unittest/unittest.dart';

import 'package:shelf/shelf_core.dart';
import 'package:shelf/src/shelf_handlers/file_handler.dart';

import '../../shared.dart';

void main() {
  var dir = new Directory(_TEST_DIR);
  assert(dir.existsSync());

  Shelf shelf;

  setUp(() {
    shelf = new Shelf()
      .add(fileHandler(_TEST_DIR));
  });

  group('basic', () {

    group('roots fail by default', () {
      for(var path in _DIRS) {
        test('path "$path"', () {
          return shelf.handleRequest(new RequestMock(path: path))
              .then((ShelfResponse response) {
                expect(response.code, 404);
                expect(response.body, 'Could not find a resource for "$path"');
              });
        });
      }
    });

    group('files succeed by default', () {
      for(var path in _FILES) {
        for(var path in [path, '/$path']) {
          test('path "$path"', () {
            return shelf.handleRequest(new RequestMock(path: path))
                .then((ShelfResponse response) {
                  expect(response.code, 200);
                  expect(response.body, startsWith('<!DOCTYPE html>'));
                });
          });
        }
      }
    });


    test("files work", () {
      return shelf.handleRequest(new RequestMock())
          .then((ShelfResponse response) {
            expect(response.code, 404);
            expect(response.body, 'Could not find a resource for "/"');
          });
    });

  });

  // only GET request, maybe HEAD?

  // root file handler at a certain path

  // access non-existant files fails with 404

  // using '..' crazy blows up

  // default document?

}

const _TEST_DIR = 'test/shelf_handlers/file_handler/test_html_root';
const _DIRS = const ['', '/', 'users', 'users/'];
const _FILES = const ['index.html', 'about.html', 'users/bob.html'];
